
;=======================================================================
;	Programa exemplo 1 para o livro "Microcontroladores PIC"
;
;	Objetivo: 		Estudar rotina de tempo baseada em software
;	Funcionamento: 		Pisca o led 7 a cada 250 ms.
;				Agosto/97
;	Upgrade:		Maio/1999. Adaptado a placa PicLab 3
;=======================================================================
	list p=16F84		; processador 16F84

	__config B'0011111111110001' 
				; CP off, DATA off, WD off, PWT on, XT


	radix	dec		; define padrao DECIMAL para valores sem identificacao
	include	<P16F84.INC>	; anexa o arquivo que contem as definicoes do 16F84
;..........................................................................
;		Tabela de definicoes de RAM e constantes.
W	equ	0		; referencia a W
Led7	equ	7		; led 7 esta em RB7

tempo	equ	0CH		; define variavel auxiliar tempo na RAM 0CH. (primeira posicao)
X	equ	0DH
Y	equ	0EH

;----------------------------- Memoria de Programa ------------------------

	org	0		; define inicio do trecho a seguir em 000

	goto	inicio		; desvia para o programa principal onde inicia pelo 
				; ajuste dos registros de controle.
				; Este desvio tem por objetivo deixar o endereco 004
				; para a rotina de interrupcao.
;.............................. INICIO DA ROTINA DE INTERRUPCAO ..........
	org	4		

	nop			; primeira instrucao da interrupcao.
	retfie			; retorna da interrupcao.

;.............................. INICIO DO PROGRAMA ......................

inicio:				; este endereco e' o imediatamente apos o fim da interrupcao

	movlw	B'00000000'	; W = 0
	movwf	INTCON		; INTCON = 0. Nenhuma interr habilitada.

	bsf	STATUS,RP0	; Seleciona banco 1 para option e tris
	movlw	B'11011111'	;
	movwf	OPTION_REG	; Registro OPTION ajusta timer 0 clock interno
				; e prescaler p/ Watch Dog (nao usamos)
	movlw	B'11111111'	; W = 255
	movwf	TRISA		; TRISA = ffH. PortA toda entrada
	movlw	B'01111111'	; W = 0
	movwf	TRISB		; TRISB = 7fh. PortB.7 saida, demais entrada

	bcf	STATUS,RP0	; volto ao banco 0. Padrao do reset



;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
principal:			;inicio do programa principal.

	bsf	PORTB,Led7 	; fazendo PORTB pino 7 = 1 acende o led 7
	call	ms250		; espera 250 ms
	bcf	PORTB,Led7 	; fazendo PORTB pino 7 = 0 apaga o led 7
	call	ms250		; espera 250 ms

	goto 	principal

;...............................Rotina de tempo de 250 ms.
ms250:				; - O CALL para a rotina leva 2 us.
	movlw	250		;+1	
	movwf	tempo		;+1	total 1 = 4 us

ms250a:
	movlw 	248           	;+1
	movwf	X           	;+1	total 2 = 2 us

ms250b:			
	nop               		;1
	decfsz 	X        		;1	248 x 4 us + (1 us nop + 2 us quando decfsz  da 0)
	goto  	ms250b     	;2	da um tempo total 3 = 995 us

	decfsz	tempo		;1	1 us pq tempo > 0 + 2 us goto da um total 4 =  3 us
	goto 	ms250a       	;2	250 x (total 2 + total 3 + total 4) + 2 us quando tempo = 0
				;	 = total 5 = 250.002 us 

	return			; delay = total 1 + total 5 + 2 us return = 250.008 us

	END			; fim do programa fonte
