;=====================================================================
;	Programa exemplo 2 para o livro "Microcontroladores PIC"
;
;	Objetivo: 		Estudar instrucoes de deslocamento
;	Funcionamento: 		Pisca os leds 7 a 0 nesta ordem e repete.
;				Agosto/97
;	Upgrade:		Leds agora acendem em 1 para Placa Piclab 3
;=====================================================================
	list p=16F84		; processador 16F84

	__config B'0011111111110001' 
				; CP off, DATA off, WD off, PWT on, XT

	radix	dec		; define padrao DECIMAL para valores sem identificacao
	include	<P16F84.INC>	; anexa o arquivo que contem as definicoes do 16F84

;.....................................................................................
;				Tabela de definicoes de RAM e constantes.
W	equ	0		; referencia a W
tempo	equ	0CH		; define variavel auxiliar tempo na RAM 0CH. (primeira posicao)
dt1	equ	0DH		; variavel auxiliar para tempo
X	equ	0EH		
Y	equ	0FH

;----------------------------- Memoria de Programa -----------------------------------

	org	0		; define inicio do trecho a seguir em 000

	goto	inicio		; desvia para o programa principal onde inicia pelo 
				; ajuste dos registros de controle.
				; Este desvio tem por objetivo deixar o endereco 004
				; para a rotina de interrupcao.

;.............................. INICIO DA ROTINA DE INTERRUPCAO ............................
	org	4		

	nop			; primeira instrucao da interrupcao.
	retfie			; retorna da interrupcao.

;.............................. INICIO DO PROGRAMA .........................................

inicio:				; este endereco e' o imediatamente apos o fim da interrupcao
				; AQUI ESTAMOS NO BANCO 0
	movlw	B'00000000'	; W = 0
	movwf	INTCON	; INTCON = 0. Nenhuma interr habilitada.

	clrf	PORTA		;inicializa portas A e B. Ver itens 7.1.4 e 7.2.7
	clrf	PORTB

	bsf	STATUS,RP0	; Seleciona banco 1 para option e tris

	movlw	B'11111111'	; W = 255
	movwf	TRISA		; TRISA = ffH. PortA toda entrada
	movlw	B'00000000'	; W = 0
	movwf	TRISB		; TRISB = 0. PortB toda saida

	movlw	B'11011111'	;
	movwf	OPTION_REG	; Registro OPTION ajusta timer 0 clock interno
				; e prescaler p/ Watch Dog (nao usamos)

	bcf	STATUS,RP0	; volto ao banco 0. Padrao do reset

;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
principal:			;inicio do programa principal.

	movlw	0		; W = 0
	movwf	X		; X = W = 0
	bsf	STATUS,C	; bit Carry = 1

repete:	
	rrf	X		; desloca o valor em X para a direita.
				; o bit 0 vai ao Carry e o Carry vai ao bit 7.

	btfsc	STATUS,C	; Se C = 0 pula prox inst. 
	goto 	principal	; Quando Carry = 1 acabou de deslocar 8 vezes.
				; reinicia.

	movf	X,W		; W = valor em X que sera escrito no PortB.
				; ver instrucao MOVF f,d. Se d=0 escreve em W.
				; Como atribuimos a W o valor 0, o compilador enxerga
				; a instrucao como MOVF X,0
escreve:
	movwf	PORTB		; PortB = W = X

	call	ms250		; espera 250 ms

	goto	repete		; Se Carry = 1 ainda nao deslocou 8 vezes
				; efetua novo deslocamento

;...............................Rotina de tempo de 250 ms.
ms250:
	movlw	250		;1
	movwf	tempo		;1

ms250a:
	movlw 	248           	;1
	movwf	dt1           	;1

ms250b:
	nop               		;1	
	decfsz 	dt1        	;1
	goto  	ms250b     	;2

	decfsz	tempo		;1
	goto 	ms250a       	;2

	return

	END			; fim do programa fonte
