;=======================================================================
;	Programa exemplo 6 para o livro "Microcontroladores PIC"
;
;	Objetivo: 		Estudar interrupcoes
;	Funcionamento: 		Complementa o PORTB a cada 100 us.
;				Inicia com PORTB = 01010101. A cada interrupcao temos PORTB
;				10101010 e depois 01010101 repetidamente.
;				O sinal dos pinos impares esta invertido em relacao aos pares
;	Agosto/97
;				Adaptado a placa PicLab 3.
;=======================================================================
	list p=16F84		; processador 16F84

	__config B'0011111111110001' 
				; CP off, DATA off, WD off, PWT on, XT

	radix	dec		; define padrao DECIMAL para valores sem identificacao
	include	<P16F84.INC>	; anexa o arquivo que contem as definicoes do 16F84
;..........................................................................
;		Tabela de definicoes de RAM e constantes.
W	equ	0		; referencia a W
saida	equ	7		; saida da onda quadrada em RB7 (led 7)
tempo	equ	0CH		; define variavel auxiliar tempo na RAM 0CH. (primeira posicao)
X	equ	0DH		; variaveis de uso geral
Y	equ	0EH
W2	equ	0FH		; espelho do W original. 

;----------------------------- Memoria de Programa ------------------------

	org	0		; define inicio do trecho a seguir em 000

	goto	inicio		; desvia para o programa principal.

;.............................. INICIO DA ROTINA DE INTERRUPCAO ..........
	org	4		

	movwf	W2		; salvo o W atual porque usarei o mesmo

	comf	X		; inverte os bits em X e salva no proprio X
	movf	X,W		; w =X
altera:
	movwf	PORTB		; PORTB recebe os bits complementados   

	movlw	165		; W = valor a recarregar em TMR0  
	movwf	TMR0		; timer 0 conta do valor ajustado ate o overflow
				; como quero overflow a cada 100 us, deveria ajustar inicio em 
				; 256 - 100 = 156, mas devido ao atraso de 2 ciclos ao recarregar 
				; TMR0, o atraso do atendimento da interrupcao e o tempo
				; das instrucoes, uso o valor  acima.
	bcf	INTCON,T0IF	; reabilito interrupcao timer 0 antes de voltar
	movf	W2,W		; recupero W original
	retfie			; retorna da interrupcao.

;.............................. INICIO DO PROGRAMA ......................

inicio:				; este endereco e' o imediatamente apos o fim da interrupcao

	movlw	B'10000000'	; GIE = 1. Ver item 5.3.3
	movwf	INTCON		; Interrupcoes habilitadas.
				; a interrupcao do timer 0 liberarei depois.
				; LEMBRE-SE QUE O TIMER 0 SEMPRE ESTA RODANDO

	bsf	STATUS,RP0	; Seleciona banco 1 para option e tris
	movlw	B'11011111'	;
	movwf	OPTION_REG	; Registro OPTION ajusta timer 0 clock interno
				; e prescaler p/ Watch Dog (nao usamos)
	movlw	B'11111111'	; W = 255
	movwf	TRISA		; TRISA = ffH. PortA toda entrada
	movlw	B'00000000'	; W = 0
	movwf	TRISB		; TRISB = 0. PortB toda saida

	bcf	STATUS,RP0	; volto ao banco 0. Padrao do reset

;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . 
principal:			;inicio do programa principal.

	movlw	55H		; W = 01010101
	movwf	X
	movwf	PORTB		; PORTB inicializa com 01010101

	movlw	165		; primeira carga do registro TMR0
	movwf	TMR0		

	bcf	INTCON,T0IF	; retiro eventual pedido pendente
	bsf	INTCON,T0IE	; habilita interrupcao timer 0

loop:
	goto 	loop		; fica aqui indefinidamente.
				; ESTE PROGRAMA BASICAMENTE E' A PROPRIA
				; INTERRUPCAO DO TIMER 0.

	END			; fim do programa fonte
				




